#ifndef _M3DMATH_H_
#define _M3DMATH_H_

// systemove kniznice
#include <math.h>
#include <vector>

#ifdef _DEBUG
#ifdef __GNUC__
#define ASSERT(c) if(!(c)) { asm("int $3"); }
#else
#define ASSERT(c) if(!(c)) { __asm int 3 }
#endif
#else
#define ASSERT(c) if(c) {}
#endif

#ifdef WIN32
#define strcasecmp _stricmp
#define PATHDIV '\\'
#else
#define __cdecl
#define _vsnprintf vsnprintf
#define PATHDIV '/'
#endif

// konstanty
#define PI    (3.141592653589f)
#define PI180 (0.017453292519f)
#define PI2   (1.570796327f)
#define RAD   (PI/180.0f)
#define DEG2RAD(a) (PI/180.0f*(a))
#define RAD2DEG(a) (180.0f/PI*(a))

// druha mocnina
#define SQR(a) ((a)*(a))

// nahodne cislo z intervalu <min;max>
#define RAND(min,max) ((float) rand()/(float) RAND_MAX*((float) max-(float) min)+(float) min)
// nahodne cislo z intervalu <-1.0;1.0>
#define FRAND (((float) rand()-(float) rand())/RAND_MAX)
// pseudo-nahodne cislo
#define PRAND(s,x) ((float) (((((unsigned int) (s))*1103515245+12345)>>16)%(x)))

// orazanie cisla do intervalu <min;max>
#define CLAMP(a,min,max) a=(a<min?min:a<max?a:max);

// makra pre cykly
#define loop(a,b) for (int a=0;a<(int) (b);a++)
#define loopi(m) loop(i,m)
#define loopj(m) loop(j,m)
#define loopk(m) loop(k,m)
#define loopl(m) loop(l,m)
#define loopv(v) for (int i=0;i<(v).length();i++)
// makra pre rusenie ukazovatelov
#define DELETEP(p) if(p) { delete   p; p=0; }
#define DELETEA(p) if(p) { delete[] p; p=0; }

// vymena hodnot 2 premennych
template<class T> static inline void swap(T &a,T &b) { T t=a; a=b; b=t; }
// urcenie premennej s vacsou hodnotou
template<class T> static inline T maxV(T a,T b) { return(a > b ? a : b); }
// urcenie premennej s mensou hodnotou
template<class T> static inline T minV(T a,T b) { return(a < b ? a : b); }

// 2-rozmerny vektor
struct tVec2 {
 float x,y;
};

// 3-rozmerny vektor
class tVec3 {
 public:

  // udaje

  union {
   struct {
    float x,y,z;
   };
   float v[3];
  };

  // konstruktory

  tVec3(float a=0.0f,float b=0.0f,float c=0.0f):x(a),y(b),z(c) {}
  tVec3(const tVec3 &vec):x(vec.x),y(vec.y),z(vec.z) {}
  tVec3(const tVec3 &a,const tVec3 &b):x(b.x-a.x),y(b.y-a.y),z(b.z-a.z) {}
  explicit tVec3(int a):x(a),y(a),z(a) {}
  explicit tVec3(float a):x(a),y(a),z(a) {}
  tVec3(int v[3]):x(v[0]),y(v[1]),z(v[2]) {}
  tVec3(float *v):x(v[0]),y(v[1]),z(v[2]) {}
  tVec3(float yaw,float pitch):x(sinf(yaw)*cosf(pitch)),y(-cosf(yaw)*cosf(pitch)),z(sinf(pitch)) {}

  // operatory a funkcie

  // index
  //float &operator[](const long IDX) {
  // return(*((&x)+IDX));
  //}
  float &operator[](int i)       { return v[i]; }
  float  operator[](int i) const { return v[i]; }
  // priradenie hodnot prvkom vektora
  const tVec3 &operator=(const tVec3 &VEC) {
   x=VEC.x; y=VEC.y; z=VEC.z;
   return(*this);
  }
  // rovnost vektorov
  const bool operator==(const tVec3 &VEC) const { return((x==VEC.x)&&(y==VEC.y)&&(z==VEC.z)); }
  // nerovnost vektorov
  const bool operator!=(const tVec3 &VEC) const { return(!(*this==VEC)); }
  //bool operator!=(const tVec3 &VEC) const { return((x!=VEC.x)||(y!=VEC.y)||(z!=VEC.z)); }
  // priradenie hodnot prvkom vektora
  void set(float SX,float SY,float SZ) { x=SX; y=SY; z=SZ; }
  tVec3 &set(int i,float f) { v[i]=f; return(*this); }
  // scitanie vektorov
  const tVec3 operator+(const tVec3 &VEC) const { return(tVec3(x+VEC.x,y+VEC.y,z+VEC.z)); }
  tVec3 &add(const tVec3 &VEC) { x+=VEC.x; y+=VEC.y; z+=VEC.z; return(*this); }
  // pricitanie skalara k prvkom vektora
  tVec3 &add(float f) { x+=f; y+=f; z+=f; return(*this); }
  // pricitanie k vektoru
  const tVec3 &operator+=(const tVec3 &VEC) { x+=VEC.x; y+=VEC.y; z+=VEC.z; return(*this); }
  // odcitanie vektorov
  const tVec3 operator-(const tVec3 &VEC) const { return(tVec3(x-VEC.x,y-VEC.y,z-VEC.z)); }
  tVec3 &sub(const tVec3 &VEC) { x-=VEC.x; y-=VEC.y; z-=VEC.z; return(*this); }
  // odcitanie skalara od prvkov vektora
  tVec3 &sub(float f) { x-=f; y-=f; z-=f; return(*this); }
  // odcitanie od vektora
  const tVec3 &operator-=(const tVec3 &VEC) { x-=VEC.x; y-=VEC.y; z-=VEC.z; return(*this); }
  // nasobenie skalarom
  const tVec3 &operator*=(const float S) { x*=S; y*=S; z*=S; return(*this); }
  // delenie skalarom
  const tVec3 &operator/=(const float S) { const float recip=1.0f/S; x*=recip; y*=recip; z*=recip; return(*this); }
  // nasobenie skalarom
  const tVec3 operator*(const float S) const { return(tVec3(x*S,y*S,z*S)); }
  // nasobenie skalarom
  friend const tVec3 operator*(const float S,const tVec3 &VEC) { return(VEC*S); }
  tVec3 &mul(float f) { x*=f; y*=f; z*=f; return(*this); }
  // delenie skalarom
  const tVec3 operator/(const float S) const { const float recip=1.0f/S; return(tVec3(recip*x,recip*y,recip*z)); }
  tVec3 &div(float f) { x/=f; y/=f; z/=f; return(*this); }
  // nasobenie vektorov
  const tVec3 operator*(const tVec3 &VEC) const { return(tVec3(x*VEC.x,y*VEC.y,z*VEC.z)); }
  // negacia kladneho vektora
  const tVec3 operator+() const { return(tVec3(*this)); }
  tVec3 &neg() { return(mul(-1.0f)); }
  // negacia zaporneho vektora
  const tVec3 operator-() const { return(tVec3(-x,-y,-z)); }

  // funkcie

  // vektorovy sucin
  const tVec3 crossP(const tVec3 &VEC) const { return(tVec3(y*VEC.z-z*VEC.y,z*VEC.x-x*VEC.z,x*VEC.y-y*VEC.x)); }
  tVec3 &cross(const tVec3 &VEC1,const tVec3 &VEC2) {
   x=VEC1.y*VEC2.z-VEC1.z*VEC2.y; y=VEC1.z*VEC2.x-VEC1.x*VEC2.z; z=VEC1.x*VEC2.y-VEC1.y*VEC2.x;
   return(*this);
  }
  // skalarny sucin
  const float dotP(const tVec3 &VEC) const { return(x*VEC.x+y*VEC.y+z*VEC.z); }
  float dot(const tVec3 &VEC) const { return(x*VEC.x+y*VEC.y+z*VEC.z); }
  // dlzka vektora
  const float Vlength() const { return((float) sqrt(x*x+y*y+z*z)); }
  // (dlzka vektora)^2
  float squaredlen() const { return(x*x+y*y+z*z); }
  float magnitude() const { return(sqrtf(squaredlen())); }
  // jednotkovy vektor
  const tVec3 Vunit() const { return((*this)/Vlength()); }
  // normalizacia vektora
  void Vnormalize() { (*this)/=Vlength(); }
  tVec3 &normalize() { div(magnitude()); return(*this); }
  // uhol medzi vektormi
  inline const float Vangle(const tVec3 &VEC) const { return(acosf(dotP(VEC))); }
  // priemet vektora do roviny definovanej normalou
  const tVec3 Vreflect(const tVec3 &N) const { return(*this-N*2.0f*dotP(N)); }
  tVec3 &reflect(const tVec3 &N) { float k=2*dot(N); x-=k*N.x; y-=k*N.y; z-=k*N.z; return(*this); }
  // rotacia vektora okolo normaly
  const tVec3 Vrotate(const float A,const tVec3 &N) const {
   const float cosine=cosf(A);
   const float sine=sinf(A);
   return(tVec3((*this)*cosine+((N*(*this))*(1.0f-cosine))*N+(crossP(N))*sine));
  }
  // nulovy vektor?
  bool iszero() const { return(x==0&&y==0&&z==0); }
  // normalizovany vektor?
  bool isnormalized() const { float m=squaredlen(); return(m>0.99f&&m<1.01f); }
 // vzdialenost dvoch vektorov
 float dist(const tVec3 &VEC) const { tVec3 t; return(dist(VEC,t)); }
 float dist(const tVec3 &VEC1,tVec3 &VEC2) const { VEC2=(*this); VEC2.sub(VEC1); return(VEC2.magnitude()); }
 // reject
 bool reject(const tVec3 &VEC,float max) { return(x>VEC.x+max||x<VEC.x-max||y>VEC.y+max||y<VEC.y-max); }
 // rescale
 void rescale(float K) { float mag=magnitude(); if (mag>1e-6f) mul(K/mag); }
 // project
 tVec3 &project(const tVec3 &N) { float k=dot(N); x-=k*N.x; y-=k*N.y; z-=k*N.z; return(*this); }
 tVec3 &projectxydir(const tVec3 &N) { if (N.z) z=-(x*N.x/N.z+y*N.y/N.z); return(*this); }
 tVec3 &projectxy(const tVec3 &N) {
  float m=squaredlen(),k=dot(N);
  projectxydir(N);
  rescale(sqrtf(maxV(m-k*k,0.0f)));
  return(*this);
 }
 tVec3 &projectxy(const tVec3 &N,float threshold) {
  float m=squaredlen(),k=minV(dot(N),threshold);
  projectxydir(N);
  rescale(sqrtf(maxV(m-k*k,0.0f)));
  return(*this);
 }
 // lerp
 void lerp(const tVec3 &VEC1,const tVec3 &VEC2,float t) {
  x=VEC1.x*(1-t)+VEC2.x*t; y=VEC1.y*(1-t)+VEC2.y*t; z=VEC1.z*(1-t)+VEC2.z*t;
 }
 // rotacia okolo osi X Y Z
 void rotate_around_x(float A) { *this=tVec3(x,                  cosf(A)*y-sinf(A)*z,cosf(A)*z+sinf(A)*y); }
 void rotate_around_y(float A) { *this=tVec3(cosf(A)*x-sinf(A)*z,y,                  cosf(A)*z+sinf(A)*x); }
 void rotate_around_z(float A) { *this=tVec3(cosf(A)*x-sinf(A)*y,cosf(A)*y+sinf(A)*x,z); }
 // rotacia okolo osi O
 void rotate(float A,const tVec3 &O) {
  float c=cosf(A),s=sinf(A);
  rotate(c,s,O);
 }
 void rotate(float C,float S,const tVec3 &O) {
  *this=tVec3(x*(O.x*O.x*(1-C)+C)     + y*(O.x*O.y*(1-C)-O.z*S) + z*(O.x*O.z*(1-C)+O.y*S),
              x*(O.y*O.x*(1-C)+O.z*S) + y*(O.y*O.y*(1-C)+C)     + z*(O.y*O.z*(1-C)-O.x*S),
              x*(O.x*O.z*(1-C)-O.y*S) + y*(O.y*O.z*(1-C)+O.x*S) + z*(O.z*O.z*(1-C)+C));
 }
 // orthogonal
 void orthogonal(const tVec3 &D) {
  int i=fabs(D.x)>fabs(D.y)?(fabs(D.x)>fabs(D.z)?0:2):(fabs(D.y)>fabs(D.z)?1:2);
  v[i]=D[(i+1)%3]; v[(i+1)%3]=-D[i]; v[(i+2)%3]=0;
 }
 void orthonormalize(tVec3 &S,tVec3 &T) const {
  S.sub(tVec3(*this).mul(dot(S)));
  T.sub(tVec3(*this).mul(dot(T))).sub(tVec3(S).mul(S.dot(T)));
 }
 // dist
 template<class T> float dist_to_bb(const T &min,const T &max) const {
  float sqrdist=0;
  loopi(3) {
   if (v[i]<min[i]) { float delta=v[i]-min[i]; sqrdist+=delta*delta; }
   else if (v[i]>max[i]) { float delta=max[i]-v[i]; sqrdist+=delta*delta; }
  }
  return(sqrtf(sqrdist));
 }
 template<class T,class S> float dist_to_bb(const T &o,S size) const {
  return(dist_to_bb(o,T(o).add(size)));
 }
};

// 4-rozmerny vektor
struct tVec4 {
 float x,y,z,w;
};

// matica 4x4
class tMatrix4 {
public:
 // 4x4=16 prvkova matica
 float m[16];

 // konstruktory
 tMatrix4() {
  m[0]=1.0f; m[4]=0.0f; m[8] =0.0f; m[12]=0.0f;
  m[1]=0.0f; m[5]=1.0f; m[9] =0.0f; m[13]=0.0f;
  m[2]=0.0f; m[6]=0.0f; m[10]=1.0f; m[14]=0.0f;
  m[3]=0.0f; m[7]=0.0f; m[11]=0.0f; m[15]=1.0f;
 }
 tMatrix4(const tMatrix4 &M) {
  (*this)=M;
 }

 float *mat() {
  return(m);
 }

 // priradenie prvkov matici
 void set(const float M[16]) {
  m[0]=M[0]; m[4]=M[4]; m[8] =M[8];  m[12]=M[12];
  m[1]=M[1]; m[5]=M[5]; m[9] =M[9];  m[13]=M[13];
  m[2]=M[2]; m[6]=M[6]; m[10]=M[10]; m[14]=M[14];
  m[3]=M[3]; m[7]=M[7]; m[11]=M[11]; m[15]=M[15];
 }

 // index
 float &operator[](const int IDX) {
  return m[IDX];
 }
 // jednotkova matica
 void identity() {
  m[0]=1.0f; m[4]=0.0f; m[8] =0.0f; m[12]=0.0f;
  m[1]=0.0f; m[5]=1.0f; m[9] =0.0f; m[13]=0.0f;
  m[2]=0.0f; m[6]=0.0f; m[10]=1.0f; m[14]=0.0f;
  m[3]=0.0f; m[7]=0.0f; m[11]=0.0f; m[15]=1.0f;
 }
 // posun
 void translate(float X,float Y,float Z) {
  m[12]+=X; m[13]+=Y; m[14]+=Z;
 }
 // rotacia
 void rotate(float X,float Y,float Z) {
  float cx,cy,cz,sx,sy,sz;
  tMatrix4 mat;

  cx=(float) cos(X); sx=(float) sin(X);
  cy=(float) cos(Y); sy=(float) sin(Y);
  cz=(float) cos(Z); sz=(float) sin(Z);

  mat.m[0] =cy*cz;
  mat.m[1] =cy*sz;
  mat.m[2] =-sy;
  mat.m[4] =(sx*sy*cz)-(cx*sz);
  mat.m[5] =(sx*sy*sz)+(cx*cz);
  mat.m[6] =sx*cy;
  mat.m[8] =(cx*sy*cz)+(sx*sz);
  mat.m[9] =(cx*sy*sz)-(sx*cz);
  mat.m[10]=cx*cy;

  (*this)=(*this)*mat;
 }
 // nasobenie matic, vysledok sa ulozi do povodnej matice
 const tMatrix4 operator*=(const tMatrix4 M) {
  m[0] =m[0]*M.m[0] +m[4]*M.m[1] +m[8] *M.m[2];
  m[1] =m[1]*M.m[0] +m[5]*M.m[1] +m[9] *M.m[2];
  m[2] =m[2]*M.m[0] +m[6]*M.m[1] +m[10]*M.m[2];
  m[3] =m[3]*M.m[0] +m[7]*M.m[1] +m[11]*M.m[2];

  m[4] =m[0]*M.m[4] +m[4]*M.m[5] +m[8] *M.m[6];
  m[5] =m[1]*M.m[4] +m[5]*M.m[5] +m[9] *M.m[6];
  m[6] =m[2]*M.m[4] +m[6]*M.m[5] +m[10]*M.m[6];
  m[7] =m[3]*M.m[4] +m[7]*M.m[5] +m[11]*M.m[6];

  m[8] =m[0]*M.m[8] +m[4]*M.m[9] +m[8] *M.m[10];
  m[9] =m[1]*M.m[8] +m[5]*M.m[9] +m[9] *M.m[10];
  m[10]=m[2]*M.m[8] +m[6]*M.m[9] +m[10]*M.m[10];
  m[11]=m[3]*M.m[8] +m[7]*M.m[9] +m[11]*M.m[10];

  m[12]=m[0]*M.m[12]+m[4]*M.m[13]+m[8] *M.m[14];
  m[13]=m[1]*M.m[12]+m[5]*M.m[13]+m[9] *M.m[14];
  m[14]=m[2]*M.m[12]+m[6]*M.m[13]+m[10]*M.m[14];
  m[15]=m[3]*M.m[12]+m[7]*M.m[13]+m[11]*M.m[14];

  return(*this);
 }
 // nasobenie matic, vysledok sa ulozi do novej matice
 const tMatrix4 operator*(const tMatrix4 M) {
  tMatrix4 mat;

  mat.m[0] =m[0]*M.m[0] +m[4]*M.m[1] +m[8] *M.m[2];
  mat.m[1] =m[1]*M.m[0] +m[5]*M.m[1] +m[9] *M.m[2];
  mat.m[2] =m[2]*M.m[0] +m[6]*M.m[1] +m[10]*M.m[2];
  mat.m[3] =m[3]*M.m[0] +m[7]*M.m[1] +m[11]*M.m[2];

  mat.m[4] =m[0]*M.m[4] +m[4]*M.m[5] +m[8] *M.m[6];
  mat.m[5] =m[1]*M.m[4] +m[5]*M.m[5] +m[9] *M.m[6];
  mat.m[6] =m[2]*M.m[4] +m[6]*M.m[5] +m[10]*M.m[6];
  mat.m[7] =m[3]*M.m[4] +m[7]*M.m[5] +m[11]*M.m[6];

  mat.m[8] =m[0]*M.m[8] +m[4]*M.m[9] +m[8] *M.m[10];
  mat.m[9] =m[1]*M.m[8] +m[5]*M.m[9] +m[9] *M.m[10];
  mat.m[10]=m[2]*M.m[8] +m[6]*M.m[9] +m[10]*M.m[10];
  mat.m[11]=m[3]*M.m[8] +m[7]*M.m[9] +m[11]*M.m[10];

  mat.m[12]=m[0]*M.m[12]+m[4]*M.m[13]+m[8] *M.m[14]+m[12];
  mat.m[13]=m[1]*M.m[12]+m[5]*M.m[13]+m[9] *M.m[14]+m[13];
  mat.m[14]=m[2]*M.m[12]+m[6]*M.m[13]+m[10]*M.m[14]+m[14];
  mat.m[15]=m[3]*M.m[12]+m[7]*M.m[13]+m[11]*M.m[14]+m[15];

  return(mat);
 }
 // nasobenie vektora maticou
 const tVec3 operator*(const tVec3 VEC) {
  float a,b,c;
  a=VEC.x*m[0]+VEC.y*m[4]+VEC.z*m[8] +m[12];
  b=VEC.x*m[1]+VEC.y*m[5]+VEC.z*m[9] +m[13];
  c=VEC.x*m[2]+VEC.y*m[6]+VEC.z*m[10]+m[14];
  return(tVec3(a,b,c));
 }
};

// trieda premennych a funkcii (objektov) pre pracu s matematickymi vypoctami
class m3dmath {
public:
 m3dmath();          // konstruktor
 virtual ~m3dmath(); // destruktor

 // absolutna hodnotu skalara
 float myAbs(float N);
 // vypocet prvkov 3-rozmerneho vektora
 tVec3 Vector(tVec3 P1,tVec3 P2);
 // sucet vektorov
 tVec3 AddVector(tVec3 V1,tVec3 V2);
 // delenie vektora skalarom
 tVec3 DivVectorByScalar(tVec3 V,float S);
 // skalarny sucin dvoch vektorov
 float Dot(tVec3 V1,tVec3 V2);
 // vektorovy sucin dvoch vektorov
 tVec3 Cross(tVec3 V1,tVec3 V2);
 // normalizacia vektora
 tVec3 Normalize(tVec3 N);
 // vzdialenost dvoch bodov
 float Distance(tVec3 P1,tVec3 P2);

 // kolizia luca s trojuholnikom
 bool RayTriCollision(tVec3 P,tVec3 L,tVec3 N,float D,
                      tVec4 C1,tVec4 C2,tVec4 C3);
 // vrati bod na usecke P1-P2, ktory je najblizsie k bodu P3
 tVec3 PointOnLine(tVec3 P1,tVec3 P2,tVec3 P3);
 // test vzajomnej polohy bodu a trojuholnika
 bool PointInTriangle(tVec3 IP,tVec4 C1,tVec4 C2,tVec4 C3);
 // test vzajomnej polohy gule a trojuholnika
 bool SphereInTriangle(tVec4 T,tVec3 C,float R);
 // test vzajomnej polohy gule a hran trojuholnika
 bool SphereInEdge(tVec3 T1,tVec3 T2,tVec3 T3,tVec3 C,float R);

 tVec3 ipoint; // priesecnik
private:
 tVec3 vector_result; // vektor
 float scalar_result; // skalar
 float recip;         // reciprocna hodnota skalara

 float magnitude; // velkost

 tVec3 v,vA,vB; // pomocne vektory
 float d,m;     // pomocne premenne
};

// 1D zoznam
template <class T> struct tVector
{
 // udaje

 static const int MINSIZE=8;

 T *buf;
 int alen,ulen;

 // konstruktory

 tVector():buf(NULL),alen(0),ulen(0) { }
 tVector(const tVector &v):buf(NULL),alen(0),ulen(0) { *this = v; }
 ~tVector() { setsize(0); if (buf) delete[] (unsigned char *) buf; }

 // operatory

 tVector<T> &operator=(const tVector<T> &v) {
  setsize(0);
  if (v.length()>alen) vrealloc(v.length());
  loopv(v) add(v[i]);
  return(*this);
 }
 T &operator[](int i) { ASSERT(i>=0&&i<ulen); return(buf[i]); }
 const T &operator[](int i) const { ASSERT(i>=0&&i<ulen); return(buf[i]); }

 // funkcie

 T &add(const T &x) {
  if (ulen==alen) vrealloc(ulen+1);
  new (&buf[ulen]) T(x);
  return buf[ulen++];
 }
 T &add() {
  if (ulen==alen) vrealloc(ulen+1);
  new (&buf[ulen]) T;
  return buf[ulen++];
 }
 T &dup() {
  if (ulen==alen) vrealloc(ulen+1);
  new (&buf[ulen]) T(buf[ulen-1]);
  return buf[ulen++];
 }
 void move(tVector<T> &v) {
  if (!ulen) {
   swap(buf,v.buf);
   swap(ulen,v.ulen);
   swap(alen,v.alen);
  }
  else {
   vrealloc(ulen+v.ulen);
   if (v.ulen) memcpy(&buf[ulen],v.buf,v.ulen*sizeof(T));
   ulen+=v.ulen;
   v.ulen=0;
  }
 }
 bool inrange(size_t i) const { return(i<size_t(ulen)); }
 bool inrange(int i) const { return(i>=0&&i<ulen); }

 T &pop() { return(buf[--ulen]); }
 T &last() { return(buf[ulen-1]); }
 void drop() { buf[--ulen].~T(); }
 bool empty() const { return(ulen==0); }
 int length() const { return(ulen); }
 void setsize(int i)         { ASSERT(i<=ulen); while(ulen>i) drop(); }
 void setsizenodelete(int i) { ASSERT(i<=ulen); ulen=i; }
 void deletecontentsp() { while(!empty()) delete   pop(); }
 void deletecontentsa() { while(!empty()) delete[] pop(); }
 T *getbuf() { return(buf); }
 const T *getbuf() const { return(buf); }
 template<class ST>
 void sort(int (__cdecl *cf)(ST *,ST *),int i=0,int n=-1) {
  qsort(&buf[i],n<0?ulen:n,sizeof(T),(int (__cdecl *)(const void *,const void *)) cf);
 }
 void vrealloc(int sz) {
  int olen=alen;
  if (!alen) alen=maxV(MINSIZE,sz);
  else while (alen<sz) alen*=2;
  if (alen<=olen) return;
  unsigned char *newbuf=new unsigned char[alen*sizeof(T)];
  if (olen>0) {
   memcpy(newbuf,buf,olen*sizeof(T));
   delete[] (unsigned char *) buf;
  }
  buf=(T *) newbuf;
 }
 void advance(int sz) { ulen+=sz; }
 void remove(int i,int n) { for (int p=i+n;p<ulen;p++) buf[p-n]=buf[p]; ulen-=n; }
 T remove(int i) { T e=buf[i]; for (int p=i+1;p<ulen;p++) buf[p-1]=buf[p]; ulen--; return e; }
 template<class U>
 int find(const U &o) {
  loopi(ulen) if (buf[i]==o) return(i);
  return(-1);
 }
 void removeobj(const T &o) {
  loopi(ulen) if (buf[i]==o) remove(i--);
 }
 void replacewithlast(const T &o) {
  if (!ulen) return;
  loopi(ulen-1) if (buf[i]==o) buf[i]=buf[ulen-1];
  ulen--;
 }
 T &insert(int i,const T &e) {
  add(T());
  for (int p=ulen-1;p>i;p--) buf[p]=buf[p-1];
  buf[i]=e;
  return(buf[i]);
 }
 T *insert(int i,const T *e,int n) {
  if (ulen+n>alen) vrealloc(ulen+n);
  loopj(n) add(T());
  for (int p=ulen-1;p>=i+n;p--) buf[p]=buf[p-n];
  loopj(n) buf[i+j]=e[j];
  return(&buf[i]);
 }
 void reverse() { loopi(ulen/2) swap(buf[i],buf[ulen-1-i]); }
};

#endif
