// hlavickove subory *******************************************************
// GL kniznica
#include <GL/gl.h>
// SDL kniznica
#include <SDL/SDL.h>

// vlastny .h subor
#include "mInterface.h"

// konstruktor *************************************************************
mInterface::mInterface()
{
 // rozmery okna
 ViewPort[0]=0; ViewPort[1]=0; ViewPort[2]=0; ViewPort[3]=0;

 lighting_onoff=0;  // osvetlenie
 help_onoff=0;      // napoveda
 pause_motion=0;    // casovo zavisly pohyb
 normals_onoff=0;   // vypnutie vykreslovania normal
 polygon_mode=0;    // vyplnene trojuholniky
 draw_mode=1;       // strom je vykresleny

 // identifikator pouziteho AVATARa
 Aid=0;

 // pomocne premenne
 t_a=0; t_b=0; t_c=0;

 // stavy OpenGL stavoveho stroja
 ogl_State[0]=0; ogl_State[1]=0;

 // strom
 my_Tree=NULL;
 // trava
 my_Grass=NULL;
}

// destruktor **************************************************************
mInterface::~mInterface()
{
 if (my_Grass) delete my_Grass;
 if (my_Tree) delete my_Tree;
}

// inicializacia OpenGL - nastavenie parametrov pre tvorbu OpenGL sceny ****
int mInterface::ogl_Initialize(void)
{
 // nastavenie 2 vseobecnych materialov
 for (t_a=0;t_a<2;t_a++) {
  ogl_Material[t_a].amb[0]=0.1f; ogl_Material[t_a].amb[1]=0.1f;
  ogl_Material[t_a].amb[2]=0.1f; ogl_Material[t_a].amb[3]=1.0f;
  ogl_Material[t_a].dif[0]=0.8f; ogl_Material[t_a].dif[1]=0.8f;
  ogl_Material[t_a].dif[2]=0.8f; ogl_Material[t_a].dif[3]=1.0f;
  ogl_Material[t_a].spe[0]=1.0f; ogl_Material[t_a].spe[1]=1.0f;
  ogl_Material[t_a].spe[2]=1.0f; ogl_Material[t_a].spe[3]=1.0f;
  ogl_Material[t_a].amb_dif[0]=0.25f; ogl_Material[t_a].amb_dif[1]=0.25f;
  ogl_Material[t_a].amb_dif[2]=0.25f; ogl_Material[t_a].amb_dif[3]=1.0f;
  ogl_Material[t_a].emi[0]=0.0f; ogl_Material[t_a].emi[1]=0.0f;
  ogl_Material[t_a].emi[2]=0.0f; ogl_Material[t_a].emi[3]=1.0f;
  ogl_Material[t_a].shi=128.0f;
  ogl_Material[t_a].transp=0.0f;
  ogl_Material[t_a].sillum[0]=0.0f; ogl_Material[t_a].sillum[1]=0.0f;
  ogl_Material[t_a].sillum[2]=0.0f; ogl_Material[t_a].sillum[3]=1.0f;
 }
 // nastavenie 2 vseobecnych svetiel
 for (t_a=0;t_a<2;t_a++) {
  ogl_Light[t_a].amb[0]=0.1f; ogl_Light[t_a].amb[1]=0.1f;
  ogl_Light[t_a].amb[2]=0.1f; ogl_Light[t_a].amb[3]=1.0f;
  ogl_Light[t_a].dif[0]=0.9f; ogl_Light[t_a].dif[1]=0.9f;
  ogl_Light[t_a].dif[2]=0.9f; ogl_Light[t_a].dif[3]=1.0f;
  ogl_Light[t_a].spe[0]=0.9f; ogl_Light[t_a].spe[1]=0.9f;
  ogl_Light[t_a].spe[2]=0.9f; ogl_Light[t_a].spe[3]=1.0f;
  ogl_Light[t_a].pos[0]=0.0f; ogl_Light[t_a].pos[1]=15.0f;
  ogl_Light[t_a].pos[2]=0.0f; ogl_Light[t_a].pos[3]=1.0f;
  // pos[3]=1.0f - vsesmerove svetlo, 0.0f - smerove svetlo
  ogl_Light[t_a].spot[0]=0.0f;
  ogl_Light[t_a].spot[1]=0.0f;
  ogl_Light[t_a].spot[2]=-1.0f;
 }

 // STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ --

 // nastavenie OpenGL zasobnika farieb a hlbky *****************************
 glClearColor(0.75f,0.85f,0.95f,0.0f); // zasobnik farieb bude vymazavany ciernou farbou
 //glClearColor(0.1f,0.1f,0.1f,0.0f); // zasobnik farieb bude vymazavany ciernou farbou
 glClearDepth(1.0f);                // zasobnik hlbky bude vymazavany nastavenim na 1.0f

 glDepthFunc(GL_LEQUAL);  // typ porovnavacej funkcie pre urcenie Z-vzdialenosti
 glEnable(GL_DEPTH_TEST); // zapnutie testovania hlbky

 glDisable(GL_CULL_FACE); // nastavenie poradia, v ktorom budu definovane vrcholy objektu
 glFrontFace(GL_CCW);     // reprezentovat privratene resp. odvratene plochy,
 glCullFace(GL_BACK);     // CCW - proti smeru hod. ruciciek, CW - v smere hod. ruciciek
 glEnable(GL_CULL_FACE);  // odvratene plochy sa nebudu spracovavat

 // vseobecny material objektov
 glMaterialfv(GL_FRONT,GL_AMBIENT,ogl_Material[0].amb);
 glMaterialfv(GL_FRONT,GL_DIFFUSE,ogl_Material[0].dif);
 glMaterialfv(GL_FRONT,GL_SPECULAR,ogl_Material[0].spe);
 glMaterialfv(GL_FRONT,GL_EMISSION,ogl_Material[0].emi);
 glMaterialf(GL_FRONT,GL_SHININESS,ogl_Material[0].shi);
 // farba bude mat vplyv iba na 'diffuse' komponent materialu
 glColorMaterial(GL_FRONT,GL_DIFFUSE);
 // pre Gourandove svetelne vypocty bude pouzity material priradeny objektu,
 // ktory bude ale stale ovplyvneny farbou zadavanou prikazom glColor
 glEnable(GL_COLOR_MATERIAL);

 // zapnutie gourand-jemneho tienovania
 glShadeModel(GL_SMOOTH);
 // trojuholniky budu vyplnene
 glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
 // zapnutie modulacie textur osvetlenim
 // vysledna farba bodu = farba z textury + osvetlenie
 glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
 // realnejsie perspektivne vypocty a korekcie
 glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
 glHint(GL_LINE_SMOOTH_HINT,GL_NICEST);
 glHint(GL_POLYGON_SMOOTH_HINT,GL_NICEST);

 // nastavenie parametrov 1. OpenGL svetla
 glLightfv(GL_LIGHT0,GL_AMBIENT,ogl_Light[0].amb);
 glLightfv(GL_LIGHT0,GL_DIFFUSE,ogl_Light[0].dif);
 glLightfv(GL_LIGHT0,GL_SPECULAR,ogl_Light[0].spe);
 glLightfv(GL_LIGHT0,GL_POSITION,ogl_Light[0].pos);
 glEnable(GL_LIGHT0); // ON
 // nastavenie parametrov 2. OpenGL svetla
 glLightfv(GL_LIGHT1,GL_AMBIENT,ogl_Light[1].amb);
 glLightfv(GL_LIGHT1,GL_DIFFUSE,ogl_Light[1].dif);
 glLightfv(GL_LIGHT1,GL_SPECULAR,ogl_Light[1].spe);
 glLightfv(GL_LIGHT1,GL_POSITION,ogl_Light[1].pos);
 //glEnable(GL_LIGHT1); // OFF
 if (lighting_onoff) glEnable(GL_LIGHTING);
 else glDisable(GL_LIGHTING);
 // svetlo sa pocita len pre privratene strany
 glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,0);
 // svetelne vypocty su lokalne
 glLightModelf(GL_LIGHT_MODEL_LOCAL_VIEWER,1.0f);

 // nastavenie standardnej farby a alfa hodnoty - biela, alfa 255
 glColor4f(1.0f,1.0f,1.0f,1.0f);

 // porovnavacia funkcia alpha testu
 glAlphaFunc(GL_GREATER,0.6f);

 // zapnutie texturovania
 glEnable(GL_TEXTURE_2D);

 // INICIALIZACIA SCENY INICIALIZACIA SCENY INICIALIZACIA SCENY ------------

 // vymazanie zasobnika farieb a hlbky
 glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

 fprintf(stderr,"----------------------------------------------------------\n");

 if (ogl_Texture.LoadCrosshairTexture("data/gfx/crosshair0.bmp")) {
  fprintf(stderr,"Chyba nahratia textury zameriavacieho kriza!\n"); return(1);
 }

 // nacitanie textury pre zem 0
 if (ogl_Texture.LoadTexture("data/grass.tga",T_MIPMAP|T_TRILINEAR)) {
  fprintf(stderr,"Chyba nahratia textury pre zem!\n"); return(1);
 }
 // nacitanie textury pre kmen a konare 1
 if (ogl_Texture.LoadTexture("data/branch0.tga",T_MIPMAP|T_TRILINEAR)) {
  fprintf(stderr,"Chyba nahratia textury pre kmen!\n"); return(1);
 }
 // nacitanie textury pre list 2
 if (ogl_Texture.LoadTexture("data/leaf0.tga",T_MIPMAP|T_TRILINEAR|T_TRANSPARENT)) {
  fprintf(stderr,"Chyba nahratia textury pre list!\n"); return(1);
 }
 // nacitanie textury pre travu 3
 if (ogl_Texture.LoadTexture("data/grass_h.tga",T_MIPMAP|T_TRILINEAR|T_TRANSPARENT)) {
  fprintf(stderr,"Chyba nahratia textury pre travu!\n"); return(1);
 }

 // vytvorenie stromu
 if (my_Tree==NULL) {
  my_Tree=new PlantTree;
  if (!my_Tree) {
   fprintf(stderr,"Chyba vytvorenia stromu!\n"); return(1);
  }
  else {
   // zmenou hodnot mozno ziskat velmi zaujimave a dostatocne realne vyzerajuce stromy
   my_Tree->Generate(3,1.2f,0.6f,60.0f,20.0f,0.06f,0.6f,ogl_Texture.tex_id[1],ogl_Texture.tex_id[2]);
  }
 }

 // vytvorenie travy
 if (my_Grass==NULL) {
  my_Grass=new PlantGrass;
  if (!my_Grass) {
   fprintf(stderr,"Chyba vytvorenia travy!\n"); return(1);
  }
 }

 return(0);
}

// vykreslenie OpenGL sceny ************************************************
int mInterface::ogl_Draw(void)
{
 // vymazanie zasobnika farieb a hlbky
 glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

 // zresetovanie modelovej matice
 glLoadIdentity();

 // TRANSFORMACIA SCENY PODLA NASTAVENI AKTUALNEJ KAMERY!
 // ROTACIA
 glMultMatrixf(Avatar[Aid].matrix);

 // TRANSFORMACIA SCENY PODLA NASTAVENI AKTUALNEJ KAMERY!
 // POSUN X, Y, Z
 glTranslatef(Avatar[Aid].posx,Avatar[Aid].posy,Avatar[Aid].posz);

 // transformacia polohy svetla podla polohy kamery
 //ogl_Light[0].pos[0]=-Avatar[Aid].posx;
 //ogl_Light[0].pos[1]=-Avatar[Aid].posy;
 //ogl_Light[0].pos[2]=-Avatar[Aid].posz;

 // umiestenenie svetla na novu poziciu
 glLightfv(GL_LIGHT0,GL_POSITION,ogl_Light[0].pos);

 // zem
 glBindTexture(GL_TEXTURE_2D,ogl_Texture.tex_id[0]);
 glBegin(GL_QUADS);
  glTexCoord2f(0.0f,0.0f);   glVertex3f(-20.0f,0.0f, 20.0f);
  glTexCoord2f(10.0f,0.0f);  glVertex3f( 20.0f,0.0f, 20.0f);
  glTexCoord2f(10.0f,10.0f); glVertex3f( 20.0f,0.0f,-20.0f);
  glTexCoord2f(0.0f,10.0f);  glVertex3f(-20.0f,0.0f,-20.0f);
 glEnd();

 // trava
 glPushMatrix();
 glBindTexture(GL_TEXTURE_2D,ogl_Texture.tex_id[3]);
 my_Grass->Draw(Avatar[Aid].posx,Avatar[Aid].posy,Avatar[Aid].posz,
                Avatar[Aid].targetx,Avatar[Aid].targety,Avatar[Aid].targetz);
 glPopMatrix();

 // strom
 glPushMatrix();
  glScalef(4.0f,4.0f,4.0f);
  if (draw_mode) my_Tree->Draw(Avatar[Aid].posx,Avatar[Aid].posy,Avatar[Aid].posz);
 glPopMatrix();
 glPushMatrix();
  //my_Tree->DrawSkeleton();
 glPopMatrix();

 // vykreslenie zameriavacieho kriza
 ogl_Texture.DrawCrosshair();

 // vykreslenie napovedy
 if (help_onoff) {

 }
 else {

 }

 return(0);
}

// casovo zavisla uprava objektov, resp. sceny *****************************
int mInterface::ogl_Idle(void)
{
 if (pause_motion==0) {

 }

 return(0);
}

// vyplnene trojuholniky/iba hrany *****************************************
void mInterface::change_polygon_mode(void)
{
 if (polygon_mode==0) {
  polygon_mode=1;
  // budu vykreslene hrany trojuholnikov
  glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
 }
 else {
  polygon_mode=0;
  // trojuholniky budu vyplnene
  glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
 }

 // pauza medzi stlaceniami klaves
 SDL_Delay(200);
}

// vykreslenie stromu ******************************************************
void mInterface::change_draw_mode(void)
{
 if (draw_mode==0) draw_mode=1;
 else draw_mode=0;

 // pauza medzi stlaceniami klaves
 SDL_Delay(200);
}
