// hlavickove subory *******************************************************
#include <cstdio>
#include <cstring>
#include <limits.h>
// GL kniznica
#include <GL/gl.h>

// vlastny .h subor
#include "mPlantGrass.h"

// hranica pre vykreslenie travy
#define h1 90.0f
// hranica pre animaciu travy
#define h0 30.0f
// LOD - pocet vzoriek je zavisly na vzdialenosti od pozorovatela
#define glod  34.0f
#define glody 2.0f

// max. pocet vzoriek pre zobrazenie bilboardov
#define gbillboard 2

// konstruktor *************************************************************
PlantGrass::PlantGrass()
{
 // sirka a vyska vzorky travy
 gwidth=4.0f; gheight=4.0f;
 // hustota vzoriek v hranicnom trojuholniku
 ggrid=4.0f;
 // aktualny pocet vzoriek travy
 numsamples=0;
 // plynutie casu
 timeticks=0.0f;

 // vytvorenie 1 hranicneho stvoruholnika
 ga.o=tVec3(-20.0f,0.0f,20.0f); // poloha stvoruholnika
 ga.grassquads=NULL;            // vrcholy stvoruholnika
 ga.grasssamples=NULL;          // poloha vzoriek

 // vrcholy hranicneho stvoruholnika su ulozene takto:
 // 2 ......... 3
 // . .         .
 // .     .     .
 // .         . .
 // 0 ......... 1
 ga.grassquads=new tVector<grassquad>;
 grassquad &pp=ga.grassquads->add();
 tVec3 v[4];
 v[0].set(-20.0f,0.0f, 20.0f);
 v[1].set( 20.0f,0.0f, 20.0f);
 v[2].set(-20.0f,0.0f,-20.0f);
 v[3].set( 20.0f,0.0f,-20.0f);
 memcpy(pp.v,v,sizeof(v));

 // vypocet poloh jednotlivych vzoriek travy
 if (!ga.grasssamples) {
  ga.grasssamples=new tVector<grasssample>;
  loopv(*ga.grassquads) { // pre vsetky hranicne stvoruholniky
   grassquad &pp=(*ga.grassquads)[i];
   loopk(4) v[k]=pp.v[k]; // vyber vrcholov stvoruholnika

   SetGrassSamples(&ga,&v[0]); // vypocet poloh vzoriek pre 1 trojuholnik (0->1->2)
   SetGrassSamples(&ga,&v[1]); // vypocet poloh vzoriek pre 2 trojuholnik (1->2->3)
  }
 }
}

// destruktor **************************************************************
PlantGrass::~PlantGrass()
{
 if (ga.grassquads) delete ga.grassquads;
 if (ga.grasssamples) delete ga.grasssamples;
}

// vypocet poloh jednotlivych vzoriek travy ********************************
int PlantGrass::SetGrassSamples(grassarray *ga,const tVec3 *v)
{
 unsigned int m,l,r; // stredny, lavy a pravy vrchol
 // m - vrchol s najmensou X a Z hodnotou
 if (v[1].z<v[0].z) m=v[1].z < v[2].z ? 1 : 2; else m=v[0].z < v[2].z ? 0 : 2;
 // l, r - ostatne dva vrcholy, pricom l.x < r.x
 l=(m+2)%3; r=(m+1)%3;
 if (v[l].x>v[r].x) swap(l,r);
 if (v[m].z==v[l].z) {
  if (v[l].x<=v[m].x) swap(m,l);
  swap(l,r);
 }
 // INFO: vybrate vrcholy
 //fprintf(stderr,"m: %u l: %u r: %u\n",m,l,r);

 // lava hranica pre vyplnanie trojuholnika
 // vektor laveho prirastku hodnot pre vyplnanie
 tVec3 b1=v[m],dl=v[l];
 dl.sub(b1);
 // koniec laveho prirastku
 float endl=v[l].z;
 if (dl.x==0&&dl.z==0) return(1);

 // prava hranica pre vyplnanie trojuholnika
 // vektor praveho prirastku hodnot pre vyplnanie
 tVec3 b2,dr;
 // koniec praveho prirastku
 float endr;
 // vyber vektora praveho prirastku
 if (v[m].z==v[r].z) {
  if (v[m].x==v[r].x) return(1);
  b2=v[r];
  dr=v[l]; dr.sub(b2);
  endr=v[l].z;
 }
 else {
  b2=v[m];
  dr=v[r]; dr.sub(b2);
  endr=v[r].z;
 }
 if (dr.z==0&&(dr.x==0||dl.z==0)) return(1);
 if (dr.x==0&&dl.x==0) return(1);
 // INFO: lava a prava hranica
 //fprintf(stderr,"b1: %f %f %f\n",b1.x,b1.y,b1.z);
 //fprintf(stderr,"b2: %f %f %f\n",b2.x,b2.y,b2.z);
 // INFO: vektor laveho a praveho prirastku
 //fprintf(stderr,"dl: %f %f %f\n",dl.x,dl.y,dl.z);
 //fprintf(stderr,"dr: %f %f %f\n",dr.x,dr.y,dr.z);

 bool grassadded=false;
 numsamples=0;

 // prirastky hodnot X a Z
 dx=ggrid; dz=ggrid;
 // INFO: prirastky hodnot X a Z
 //fprintf(stderr,"%f %f\n",dx,dz);
 while (1) {
  if (endl>b1.z) dz=minV(dz,endl-b1.z);
  if (endr>b2.z) dz=minV(dz,endr-b2.z);

  // prirastok lavej hranice
  b1.x+=dl.x*dz/dl.z;
  b1.y+=dl.y*dz/dl.z;
  b1.z+=dz;
  // prirastok pravej hranice
  b2.x+=dr.x*dz/dr.z;
  b2.y+=dr.y*dz/dr.z;
  b2.z+=dz;

  if (b1.z<=endl&&b2.z<=endr) {
   // p - zaciatok ciary b1->b2, dp - prirastok ciary
   tVec3 p=b1,dp=b2;
   dp.sub(b1);

   if (b1.x==b2.x&&dx==ggrid) {
    // suradnice vzorky travy
    //fprintf(stderr,"1: %f %f %f\n",p.x,p.y,p.z);
    // nova vzorka travy
    if (!numsamples++) grassadded=true;
    grasssample &pp=ga->grasssamples->add();
    pp.x=(short int) p.x; // suradnice vzorky
    pp.y=(short int) p.y;
    pp.z=(short int) p.z;
    loopk(3) pp.c[k]=255; // farba
   }
   else {
    while (!grassadded||numsamples<USHRT_MAX) {
     // vytvorenie viacerych vzoriek na ciare b1->b2
     p.x+=dx;
     p.y+=dp.y*dx/dp.x;
     p.z+=dp.z*dx/dp.x;
     if (p.x>b2.x) break;
     // suradnice vzorky travy
     //fprintf(stderr,"2: %f %f %f\n",p.x,p.y,p.z);
     // nova vzorka travy
     if (!numsamples++) grassadded=true;
     grasssample &pp=ga->grasssamples->add();
     pp.x=(short int) p.x; // suradnice vzorky
     pp.y=(short int) p.y;
     pp.z=(short int) p.z;
     loopk(3) pp.c[k]=255; // farba
    } // while
   } // else

   if (grassadded&&numsamples>=USHRT_MAX) break;
  }
  // korekcia vektora laveho prirastku
  if (b1.z>endl) {
   if (v[r].z<=endl) break; // break plati pre while
   dl=v[r]; dl.sub(v[l]);
   endl=v[r].z;
   continue; // continue plati pre while
  }
  // korekcia vektora praveho prirastku
  if (b2.z>endr) {
   if(v[l].z<=endr) break; // break plati pre while
   dr=v[l]; dr.sub(v[r]);
   endr=v[l].z;
   continue; // continue plati pre while
  }
 } // while

 return(0);
}

// vykreslenie vzoriek travy ***********************************************
int PlantGrass::RenderGrassSamples(grassarray *ga,const tVec3 &v)
{
 // hranicny stvoruholnik
 grassquad &qq=(*ga->grassquads)[0];

 // pre vsetky vzorky patriace hranicnemu stvoruholniku
 loopv(*ga->grasssamples) {
  grasssample &pp=(*ga->grasssamples)[i]; // info o konkretnej vzorke

  // poloha vzorky
  vv.x=(float) pp.x; vv.y=(float) pp.y; vv.z=(float) pp.z;
  // di - vzdialenost kamera->vzorka, tg - vektor kamera->vzorka
  di=vv.dist(cpos,tg);
  if (di>h1) continue; // ak je vzorka za hranicou vykreslenia, nevykresli sa

  // ld - vzdialenost kamera->vzorka vypocitana na zaklade LOD
  dx=vv.x-cpos.x; dy=vv.y-cpos.y; dz=vv.z-cpos.z;
  dd=sqrt(dx*dx+dz*dz);
  ld=(glod+10.0f-maxV(dd,0.0f)+glody*maxV(-dy,0.0f))/10.0f;

  // pocet vzoriek zavisly na vzdialenosti od pozorovatela
  numsamples=(unsigned int) maxV(1.0f,ld);
  // INFO: pocet vzoriek pre 1 konkretnu situaciu
  //if (vv.x==0.0f&&vv.z==20.0f) fprintf(stderr,"%u\n",numsamples);
  //numsamples=1;

  // pre vypocitany pocet vzoriek
  loopj(2*numsamples) {
   // vektor smerujuci "doprava" - sposobi natacanie vzoriek
   right.set((float) (j%2),0.0f,(float) ((j+1)%2));

   // ak je pocet vzoriek mensi ako hodnota gbillboard, right vektor sa pootoci
   // podla kamery - vzorky tak budu vzdy natocene kolmo na kameru
   if (numsamples<=gbillboard) {
    if (j%2) break; // ak pocet vzoriek = 1, vykresli sa iba 1 vzorka
    // vzorka sa natoci kolmo na kameru
    right=v; right.y=0.0f; right.rotate_around_y(90.0f*RAD);
   }
   // pseudo-nahodne otocenie vzorky okolo osi Y v intervale <-30;+30>
   right.rotate_around_y((PRAND((size_t) &pp*(j+1),2*30)-30)*RAD);

   // lavy dolny vrchol vzorky
   a1=right; a1.mul(-0.5f*gwidth);
   a1.add(vv); // umiestnenie podla polohy vzorky
   // pseudo-nahodna uprava polohy, j/2 zabezpeci posun kazdej druhej dvojice vzoriek
   a1[0]+=(float) (j/2)*ggrid/(float) numsamples-ggrid;
   a1[2]+=(float) (j/2)*ggrid/(float) numsamples-ggrid;
   // orezanie do hranicneho stvoruholnika
   // potrebne dopracovat tak, aby sa poloha vzorky porovnavala
   // s lubovolnou hranicou, nie s hranicami rovnobeznymi s osami X a Z
   if (a1.x<qq.v[0].x) a1.x=qq.v[0].x; else if (a1.x>qq.v[1].x) a1.x=qq.v[1].x;
   if (a1.z>qq.v[0].z) a1.z=qq.v[0].z; else if (a1.z<qq.v[2].z) a1.z=qq.v[2].z;
   // pravy dolny vrchol vzorky
   a2=right; a2.mul(gwidth);
   a2.add(a1); // "pricitanie" k lavemu dolnemu vrcholu vzorky

   // orezanie do hranicneho stvoruholnika
   // potrebne dopracovat tak, aby sa poloha vzorky porovnavala
   // s lubovolnou hranicou, nie s hranicami rovnobeznymi s osami X a Z
   if (a2.x<qq.v[0].x) a2.x=qq.v[0].x; else if (a2.x>qq.v[1].x) a2.x=qq.v[1].x;
   if (a2.z>qq.v[0].z) a2.z=qq.v[0].z; else if (a2.z<qq.v[2].z) a2.z=qq.v[2].z;

   // lavy horny a pravy horny vrchol
   b1=a1; b2=a2;
   b1.y+=gheight; b2.y+=gheight;

   // animacia hornych vrcholov vzoriek
   w1=0.0f; w2=0.0f;
   if (di<h0) { // ak je vzorka pred hranicou pre animaciu
    c1=b2; c1.sub(b1); c1.rotate_around_y(90.0f*RAD);
    c2=c1;
    // pseudo-nahodne cislo
    w1=PRAND((size_t) &pp*(j+1)*10,360)*RAD+timeticks;
    w1=sinf(w1)/30.0f;
    c1.mul(gheight*w1);
    b1.add(c1);
    // pseudo-nahodne cislo
    w2=PRAND((size_t) &pp*(j+1)*20,360)*RAD+timeticks;
    w2=sinf(w2)/30.0f;
    c2.mul(gheight*w2);
    b2.add(c2);
    //fprintf(stderr,"%f %f\n",w1,w2);
   }

   glColor3ubv(pp.c); // farba vzorky
   // nahodny posun textury vzorky
   dd=PRAND((size_t) &pp*(j+1)*10,512)/512.0f;
   // texturovacie suradnice a vrcholy vzorky
   glTexCoord2f(dd,0.0f); glVertex3fv(b1.v);
   glTexCoord2f(dd,1.0f); glVertex3fv(a1.v);
   glTexCoord2f(dd+gwidth*0.5f,1.0f); glVertex3fv(a2.v);
   glTexCoord2f(dd+gwidth*0.5f,0.0f); glVertex3fv(b2.v);
  } // loopj
 } // loopv

 return(0);
}

// vykreslenie travy *******************************************************
int PlantGrass::Draw(float PX,float PY,float PZ,float TX,float TY,float TZ)
{
 cpos.x=-PX; cpos.y=-PY; cpos.z=-PZ; // poloha kamery
 crot.x=-TX; crot.y=-TY; crot.z=-TZ; // natocenie kamery

 glDisable(GL_CULL_FACE); // obojstranne kreslenie
 glEnable(GL_ALPHA_TEST); // alfa priesvitnost
 glBegin(GL_QUADS);       // stvoruholniky
  RenderGrassSamples(&ga,crot); timeticks+=0.01f;
 glEnd();
 glDisable(GL_ALPHA_TEST);
 glEnable(GL_CULL_FACE);

 return(0);
}
