// hlavickove subory *******************************************************
#include <ctime>
// GL kniznica
#include <GL/gl.h>
#include <GL/glu.h>
// SDL kniznica
#include <SDL/SDL.h>

// vlastny .h subor
#include "mPlantTree.h"

// konstruktor *************************************************************
PlantTree::PlantTree()
{
 // kmen stromu - 1. cast kostry stromu
 stem=NULL;

 // reset prirastkov zmien poloh jednotlivych segmentov
 Sline.sgs[0].set(0.0f,0.0f,0.0f);
 Sline.sgs[1].set(0.0f,0.0f,0.0f);
 Sline.sgs[2].set(0.0f,0.0f,0.0f);

 // inicializacia generatora nahodnych cisiel
 srand(time(NULL));
}

// destruktor **************************************************************
PlantTree::~PlantTree()
{

}

// vytvorenie stromu *******************************************************
int PlantTree::Generate(int MAXDEPTH,
                        float LENGTH,float LENGTHMOD,
                        float ANGLE,float ANGLEMOD,
                        float BRANCHWIDTH,float LEAFSIZE,
                        GLuint BRANCHTEX,GLuint LEAFTEX)
{
 // identifikatory textur pre konare (aj kmen) a listy stromu
 branch_texture=BRANCHTEX;
 leaf_texture=LEAFTEX;
 // hlbka rekurzie
 Tmaxdepth=MAXDEPTH;
 // dlzka casti kostry stromu, koeficient kratenia dlzky
 Tlength=LENGTH; Tlengthmod=LENGTHMOD;
 // uhol natocenia casti kostry stromu, koeficient zmeny natocenia
 Tangle=ANGLE; Tanglemod=ANGLEMOD;
 // hrubka konara, velkost listu
 Tbranchwidth=BRANCHWIDTH; Tleafsize=LEAFSIZE;
 // vymazanie pola ciar - casti kostry stromu a pola listov stromu
 lines.clear();
 leaves.clear();

 mem=0;
 // vytvorenie 1. casti kostry stromu - kmen
 stem=new branch;
 if (stem==NULL) {
  fprintf(stderr,"Chyba vyhradenia pamatoveho miesta!\n"); return(1);
 }
 mem+=sizeof(branch);
 stem->start=0.0f;
 parent=false; // 1. cast kostry stromu nema "rodica"

 // vytvorenie dalsich casti kostry stromu
 TreeRecurse(stem,stem->m,
             0,
             Tlength,0.0f);
 // INFO: celkova vyhradena pamat pre kostru stromu
 //fprintf(stderr,"Vyhradena pamat: %i\n",mem);

 mem=0;
 // uvolnenie vyhradeneho pamatoveho miesta
 if (stem) {
  RecurseDelete(stem,0);
  delete stem; stem=NULL;
  mem+=sizeof(branch);
 }
 // INFO: celkova uvolnena pamat pre kostru stromu
 //fprintf(stderr,"Uvolnena pamat: %i\n",mem);

 // vytvorenie 2D stromu - billboardov
 // najskor je potrebne spravne nastavit zobrazenie
 // pouziju sa tie iste hodnoty ako v module Avatar
 // DOROBIT: tato cast by sa dala zlepsit tak, aby
 // sa 2D textura 100% podobala na 3D strom,
 // dalo by sa tak urobit pripravou textury v case, ked
 // by malo dojst k prepnutiu medzi 3D a 2D zobrazenim,
 // to nastane ak sa pozorovatel vzdiali na vzdialenost
 // THRESHOLD1, takyto pristup je vsak narocnejsi najma
 // na vypoctovy vykon, experimentovat sa vsak stale da
 glMatrixMode(GL_PROJECTION); // vyber projekcnej matice
 glLoadIdentity();            // zresetovanie projekcnej matice
 // hodnoty su nastavene "na tvrdo" z modulu Avatar a Plants
 gluPerspective(60.0f,720.0f/486.0f,0.1f,100.0f); // nastavenie perspektivneho premietania
 glViewport(0,0,720,486);                         // nastavenie viditelnej plochy okna

 glMatrixMode(GL_MODELVIEW); // vyber modelview matice
 glLoadIdentity();           // zresetovanie modelview matice
 // vytvorenie definovaneho poctu 2D textur
 for (l=0;l<SPRITES_NUM;l++) RenderSprite(l);

 return(0);
}

// vytvorenie jednotlivych casti kostry stromu *****************************
// REKURZIVNA FUNKCIA!!!
int PlantTree::TreeRecurse(branch *B,tMatrix4 M,
                           int DEPTH,
                           float LENGTH,float ANGLE)
{
 // INFO: hlbka rekurzie
 //fprintf(stderr,"%i\n",DEPTH);
 // po dosiahnuti zadanej hlbky sa rekurzia vrati o krok spat
 if (DEPTH>=Tmaxdepth) return(0);

 B->length=LENGTH; // dlzka konara - casti kostry stromu
 B->angle=ANGLE;   // natocenie konara okolo osi Z

 tMatrix4 mnew;
 // posun na startovaciu poziciu
 mnew.translate(0.0f,B->start,0.0f);
 // natocenie okolo osi Y
 mnew.rotate(0.0f,DEG2RAD(10.0f*RAND(-Tanglemod,Tanglemod)),0.0f);
 // natocenie okolo osi Z
 mnew.rotate(0.0f,0.0f,DEG2RAD(B->angle));
 // nastavenie polohy a natocenia konara - casti kostry stromu
 B->m=M=M*mnew;

 // nastavenie hodnot ciary patriacej kostre stromu
 Sline.s=B->m*tVec3(0.0f,0.0f,0.0f);      // zaciatok ciary
 Sline.e=B->m*tVec3(0.0f,B->length,0.0f); // koniec ciary
 Sline.d=DEPTH;                           // hlbka rekurzie
 Sline.l=B->length;                       // dlzka ciary
 Sline.is_parent=parent;                  // indikator existencie "rodica"
 Sline.segs=3;                            // pocet segmentov ciary
 if (DEPTH<Tmaxdepth-1) {
  // bezny konar, z ktoreho rastu dalsie konare je zlozeny z viacerych navzajom posunutych segmentov
  Sline.sgs[0].x=RAND(-0.03f,0.03f); Sline.sgs[0].y=RAND(-0.03f,0.03f); Sline.sgs[0].z=RAND(-0.03f,0.03f);
  Sline.sgs[1].x=RAND(-0.03f,0.03f); Sline.sgs[1].y=RAND(-0.03f,0.03f); Sline.sgs[1].z=RAND(-0.03f,0.03f);
  Sline.sgs[2].x=RAND(-0.03f,0.03f); Sline.sgs[2].y=RAND(-0.03f,0.03f); Sline.sgs[2].z=RAND(-0.03f,0.03f);
  lines.push_back(Sline);
 }
 else {
  // koncove casti kostry stromu - vetvy su takisto zlozene z viacerych segmentov,
  // ich vzajomne posuny su vsak mensie
  Sline.sgs[0].x=RAND(-0.02f,0.02f); Sline.sgs[0].y=RAND(-0.02f,0.02f); Sline.sgs[0].z=RAND(-0.02f,0.02f);
  Sline.sgs[1].x=RAND(-0.02f,0.02f); Sline.sgs[1].y=RAND(-0.02f,0.02f); Sline.sgs[1].z=RAND(-0.02f,0.02f);
  Sline.sgs[2].x=RAND(-0.02f,0.02f); Sline.sgs[2].y=RAND(-0.02f,0.02f); Sline.sgs[2].z=RAND(-0.02f,0.02f);
  lines.push_back(Sline);

  // DOROBIT: dopracovat tak, aby bol zaciatok listov na spravnom mieste
  // voci "rodicovskemu" konaru, koncovy konar (vetva) ma totiz segmenty
  // a listy su voci tymto posunutym segmentom nespravne naviazane,
  // nekopiruju polohu casti kostry stromu - vznikaju nepekne artefakty

  // vytvorenie 2 listov
  // poloha listov patriacich danemu konaru
  lf1.pos=Sline.s+(Sline.e-Sline.s)*RAND(0.5f,0.75f);
  lf2.pos=Sline.s+(Sline.e-Sline.s)*RAND(0.75f,1.0f);
  // natocenie listov patriacich danemu konaru
  lf1.dir=tVec3(RAND(-1.0f,1.0f),RAND(-0.1f,-0.2f),RAND(-1.0f,1.0f));
  lf2.dir=tVec3(RAND(-1.0f,1.0f),RAND(-0.1f,-0.2f),RAND(-1.0f,1.0f));
  lf1.dir.Vnormalize();
  lf2.dir.Vnormalize();
  // posun listov o 1/2 velkosti - listy budu ku konarom pripojene stopkami
  lf1.pos+=lf1.dir*Tleafsize/2.0f;
  lf2.pos+=lf2.dir*Tleafsize/2.0f;
  // ohnutie listov
  lf1.bend=tVec3(1.0f,RAND(1.2f,2.0f),1.0f);
  lf2.bend=tVec3(1.0f,RAND(1.2f,2.0f),1.0f);
  // ulozenie listov do pola
  leaves.push_back(lf1);
  leaves.push_back(lf2);
 }

 // reset "potomkov" aktualneho konara
 B->middle=NULL; B->left=NULL; B->right=NULL;

 // pre vsetky hlbky kostry stromu okrem poslednej budu
 // vytvoreny 3 "potomkovia" - stredny, lavy a pravy
 if (DEPTH<Tmaxdepth-1) {
  // stredny "potomok"
  B->middle=new branch;
  if (B->middle==NULL) {
   fprintf(stderr,"Chyba vyhradenia pamatoveho miesta!\n"); return(1);
  }
  mem+=sizeof(branch);
  B->middle->start=B->length;
  // stredny "potomok" ma "rodica" co znamena, ze 2 casti kostry stromu
  // na seba priamo nadvazuju
  parent=true;
  TreeRecurse(B->middle,M,
              DEPTH+1,
              LENGTH*Tlengthmod,
              0.0f+RAND(-Tanglemod,Tanglemod));

  // lavy a pravy "potomok"
  // DOROBIT: dopracovat tak, aby bol zaciatok konara na spravnom mieste
  // voci "rodicovskemu" konaru, pretoze konare su zlozene zo segmentov,
  // ktore su navzajom posunute a "potomkovia" tak nekopiruju polohu
  // casti kostry stromu - vznikaju diery, konare nie su spravne napojene
  B->left=new branch;
  if (B->left==NULL) {
   fprintf(stderr,"Chyba vyhradenia pamatoveho miesta!\n"); return(1);
  }
  mem+=sizeof(branch);
  B->left->start=B->length*RAND(0.5f,1.0f);
  parent=false; // "potomok" nema priameho "rodica"
  TreeRecurse(B->left,M,
              DEPTH+1,
              LENGTH*Tlengthmod,
              -(Tangle+RAND(-Tanglemod,Tanglemod)));

  B->right=new branch;
  if (B->right==NULL) {
   fprintf(stderr,"Chyba vyhradenia pamatoveho miesta!\n"); return(1);
  }
  mem+=sizeof(branch);
  B->right->start=B->length*RAND(0.5f,1.0f);
  parent=false; // "potomok" nema priameho "rodica"
  TreeRecurse(B->right,M,
              DEPTH+1,
              LENGTH*Tlengthmod,
              Tangle+RAND(-Tanglemod,Tanglemod));
 }

 return(0);
}

// uvolnenie vyhradeneho pamatoveho miesta *********************************
// REKURZIVNA FUNKCIA!!!
int PlantTree::RecurseDelete(branch *B,int DEPTH)
{
 // INFO: hlbka rekurzie
 //fprintf(stderr,"%i\n",DEPTH);
 // po dosiahnuti zadanej hlbky sa rekurzia vrati o krok spat
 if (DEPTH>=Tmaxdepth) return(0);

 RecurseDelete(B->middle,DEPTH+1);
 // zmazanie strednych "potomkov"
 if (B->middle) { delete B->middle; B->middle=NULL; mem+=sizeof(branch); }
 RecurseDelete(B->left,DEPTH+1);
 // zmazanie lavych "potomkov"
 if (B->left) { delete B->left; B->left=NULL; mem+=sizeof(branch); }
 RecurseDelete(B->right,DEPTH+1);
 // zmazanie pravych "potomkov"
 if (B->right) { delete B->right; B->right=NULL; mem+=sizeof(branch); }

 return(0);
}

// vykreslenie kostry stromu ***********************************************
void PlantTree::DrawSkeleton(void)
{
 glPushAttrib(GL_LIGHTING_BIT|GL_TEXTURE_BIT);
 glDisable(GL_LIGHTING);   // vypnutie osvetlenia
 glDisable(GL_TEXTURE_2D); // vypnutie texturovania
 // cervena farba
 glColor4f(1.0f,0.0f,0.0f,1.0f);
 // zvacsenie kostry stromu
 glScalef(3.0f,3.0f,3.0f);
 // vykreslenie ciar
 glBegin(GL_LINES);
  for (i=0;i<(signed int) lines.size();i++) {
   glVertex3f(lines[i].s.x,lines[i].s.y,lines[i].s.z);
   glVertex3f(lines[i].e.x,lines[i].e.y,lines[i].e.z);
  }
 glEnd();
 // biela farba
 glColor4f(1.0f,1.0f,1.0f,1.0f);
 // obnova OpenGL nastaveni
 glPopAttrib();
}

// vykreslenie 3D stromu ***************************************************
void PlantTree::RenderTree()
{
 // zvacsenie stromu
 glScalef(3.0f,3.0f,3.0f);

 // vykreslenie kmena a konarov
 // textura pre kmen a konare
 glBindTexture(GL_TEXTURE_2D,branch_texture);
 // postupne sa prejdu vsetky casti kostry stromu
 for (i=0;i<(signed int) lines.size();i++) {
  // velkost spodneho konca konara
  if (lines[i].d==0) botrad=lines[i].l*Tbranchwidth;
  else botrad=lines[i].l*Tbranchwidth/(float) lines[i].d;
  // velkost vrchneho konca konara
  if ((lines[i].d+1)==0) toprad=lines[i].l*Tlengthmod*Tbranchwidth;
  else toprad=lines[i].l*Tlengthmod*Tbranchwidth/(float) (lines[i].d+1);
  // vykreslenie konara
  RenderBranch(i,botrad,toprad,max(6.0f-(float) lines[i].d,3.0f));
 }

 // vykreslenie listov

 // zobrazovanie oboch stran trojuholnikov
 glDisable(GL_CULL_FACE);
 // zapnutie alpha testovania
 glEnable(GL_ALPHA_TEST);

 // textura pre listy
 glBindTexture(GL_TEXTURE_2D,leaf_texture);
 // postupne sa prejdu vsetky casti kostry stromu
 for (i=0;i<(signed int) leaves.size();i++) {
  // poloha listu
  Vpos=leaves[i].pos;

  // natocenie listu
  Vdir=leaves[i].dir;
  Vdirnorm=Vdir;
  // ohnutie listu
  Vbend=Vdirnorm*leaves[i].bend;
  Vbend.Vnormalize();

  Vdirnorm.Vnormalize();

  if (Vdirnorm==tVec3(0.0f,1.0f,0.0f) || Vdirnorm==tVec3(0.0f,-1.0f,0.0f))
   Vright=tVec3(1.0f,0.0f,0.0f);
  else Vright=Vdir.crossP(tVec3(0.0f,1.0f,0.0f));

  Vright.Vnormalize();

  // velkost listu
  Vsize=Tleafsize/2.0f;

  // OpenGL stvoruholnik - objekt pokryty texturou v podobe listu stromu
  // DOROBIT: list stromu je reprezentovany stvoruholnikom pokrytym
  // specialnou texturov v podobe konara stromu s listami, tento stvoruholnik
  // je v strede nepatrne "ohnuty", ohnutie mozno modifikovat
  // a dosiahnut tak zaujimave vysledky, pripadne list mozno zlozit
  // z viacerych stvoruholnikov co povedie k dalsiemu zvyseniu kvality
  glPushMatrix();
   glTranslatef(Vpos.x,Vpos.y,Vpos.z);
   glBegin(GL_QUADS);
    // 1/2 stvoruholnika
    PA=Vdir*(-Vsize)+Vright*(-Vsize);
    PB=Vdir*(-Vsize)+Vright*( Vsize);
    PC=Vright*( Vsize);
    PD=Vright*(-Vsize);

    glTexCoord2f(0.0f,0.0f); glVertex3f(PA.x,PA.y,PA.z);
    glTexCoord2f(1.0f,0.0f); glVertex3f(PB.x,PB.y,PB.z);
    glTexCoord2f(1.0f,0.5f); glVertex3f(PC.x,PC.y,PC.z);
    glTexCoord2f(0.0f,0.5f); glVertex3f(PD.x,PD.y,PD.z);

    // 2/2 stvoruholnika
    PA=Vright*(-Vsize);
    PB=Vright*( Vsize);
    PC=Vbend*( Vsize)+Vright*( Vsize);
    PD=Vbend*( Vsize)+Vright*(-Vsize);

    glTexCoord2f(0.0f,0.5f); glVertex3f(PA.x,PA.y,PA.z);
    glTexCoord2f(1.0f,0.5f); glVertex3f(PB.x,PB.y,PB.z);
    glTexCoord2f(1.0f,1.0f); glVertex3f(PC.x,PC.y,PC.z);
    glTexCoord2f(0.0f,1.0f); glVertex3f(PD.x,PD.y,PD.z);
   glEnd();
   glRotatef(60.0f,Vdir.x,Vdir.y,Vdir.z);
   glBegin(GL_QUADS);
    // 1/2 stvoruholnika
    PA=Vdir*(-Vsize)+Vright*(-Vsize);
    PB=Vdir*(-Vsize)+Vright*( Vsize);
    PC=Vright*( Vsize);
    PD=Vright*(-Vsize);

    glTexCoord2f(0.0f,0.0f); glVertex3f(PA.x,PA.y,PA.z);
    glTexCoord2f(1.0f,0.0f); glVertex3f(PB.x,PB.y,PB.z);
    glTexCoord2f(1.0f,0.5f); glVertex3f(PC.x,PC.y,PC.z);
    glTexCoord2f(0.0f,0.5f); glVertex3f(PD.x,PD.y,PD.z);

    // 2/2 stvoruholnika
    PA=Vright*(-Vsize);
    PB=Vright*( Vsize);
    PC=Vbend*( Vsize)+Vright*( Vsize);
    PD=Vbend*( Vsize)+Vright*(-Vsize);

    glTexCoord2f(0.0f,0.5f); glVertex3f(PA.x,PA.y,PA.z);
    glTexCoord2f(1.0f,0.5f); glVertex3f(PB.x,PB.y,PB.z);
    glTexCoord2f(1.0f,1.0f); glVertex3f(PC.x,PC.y,PC.z);
    glTexCoord2f(0.0f,1.0f); glVertex3f(PD.x,PD.y,PD.z);
   glEnd();
  glPopMatrix();
 }

 // vypnutie alpha testovania
 glDisable(GL_ALPHA_TEST);
 // odvratene strany trojuholnikov nebudu renderovane
 glEnable(GL_CULL_FACE);
}

// vykreslenie 3D konara stromu ********************************************
void PlantTree::RenderBranch(unsigned int I,float BOTRAD,float TOPRAD,float SEGS)
{
 L=lines[I];
 // vykreslenie daneho poctu segmentov konara
 for (j=0;j<L.segs;j++) {
  Vdir=L.e-L.s; // natocenie konara
  Vdirnorm=Vdir;
  Vdirnorm.Vnormalize();

  // nastavenie zaciatocnej a koncovej suradnice segmentu
  Vstart=L.s+Vdir*(float) j/(float) L.segs;
  Vend=L.s+Vdir*(float) (j+1)/(float) L.segs;
  // posun suradnic podla prirastkov zmien poloh jednotlivych segmentov
  if (j>0) Vstart+=L.sgs[j-1];
  if (j+1>0) Vend+=L.sgs[j];

  // vypocet prvkov vektora smerujuceho "doprava" a "nahor"
  if (Vdirnorm==tVec3(0.0f,1.0f,0.0f) || Vdirnorm==tVec3(0.0f,-1.0f,0.0f))
   Vright.set(1.0f,0.0f,0.0f);
  else Vright=Vdir.crossP(tVec3(0.0f,0.0f,1.0f));
  Vup=Vdir.crossP(Vright);
  Vright.Vnormalize();
  Vup.Vnormalize();

  // OpenGL pas trojuholnikov - objekt pokryty texturou v podobe konara stromu
  // vrchne vrcholy segmentov a takisto vrcholy horneho konca konara sa
  // vypocitaju pomocou sin/cos funkcii
  // vrcholy spodneho konca konara sa prevezmu z "rodicovskeho" konara
  // ak existuje, ak nie tak sa vypocitaju pomocou sin/cos funkcii
  lim=2.0f*PI; inc=2.0f*PI/SEGS; inc1=2.0f*PI/(SEGS+1);
  botrad=BOTRAD*(1.0f-(float) j/(float) L.segs)+TOPRAD*((float) j/(float) L.segs);
  toprad=BOTRAD*(1.0f-(float) (j+1)/(float) L.segs)+TOPRAD*((float) (j+1)/(float) L.segs);

  k=0;
  // ak dana cast stromu nema "rodica" vykresli sa samostatne
  if (L.is_parent==false) {
   glBegin(GL_TRIANGLE_STRIP);
    for (r=0.0f;r<=lim;r+=inc) {
     // vrchny vrchol
     Vpos=Vend+(Vright*cosf(r)*toprad+Vup*sinf(r)*toprad);
     glTexCoord2f(r/lim,(j+1)/(float) L.segs);
     glVertex3f(Vpos.x,Vpos.y,Vpos.z);
     // vrchny vrchol sa ulozi do pola pre buduce pouzitie
     if ((j+1)==L.segs) lines[I].topP[k]=Vpos;
     k++;
     // spodny vrchol
     Vpos=Vstart+(Vright*cosf(r)*botrad+Vup*sinf(r)*botrad);
     glTexCoord2f(r/lim,j/(float) L.segs);
     glVertex3f(Vpos.x,Vpos.y,Vpos.z);
    }
   glEnd();
  }
  else {
   // ak dana cast ma "rodica", jej spodne vrcholy sa prevezmu
   // z "rodicovskeho" konara, ostatne sa vypocitaju samostatne
   glBegin(GL_TRIANGLE_STRIP);
    for (r=0.0f,r1=0.0f;r<=lim;r+=inc,r1+=inc1) {
     // vrchny vrchol
     Vpos=Vend+(Vright*cosf(r)*toprad+Vup*sinf(r)*toprad);
     glTexCoord2f(r/lim,(j+1)/(float) L.segs);
     glVertex3f(Vpos.x,Vpos.y,Vpos.z);
     // vrchny vrchol sa ulozi do pola pre buduce pouzitie
     if ((j+1)==L.segs) lines[I].topP[k]=Vpos;
     // spodny vrchol sa prevezme z "rodicovskeho" konara,
     // alebo ak ide o segment tak sa vypocita samostatne
     if (j==0) { Vpos=lines[I-1].topP[k]; glTexCoord2f(r1/lim,j/(float) L.segs); }
     else { Vpos=Vstart+(Vright*cosf(r)*botrad+Vup*sinf(r)*botrad);
            glTexCoord2f(r/lim,j/(float) L.segs); }
     k++;
     // spodny vrchol
     glVertex3f(Vpos.x,Vpos.y,Vpos.z);
    }
    // pretoze pocet vrcholov "rodicovskeho" konara je o 1 vyssi
    // je potrebne doplnit 1 vrchol pre korektne uzavretie modelu
    if (j==0) {
     Vpos=lines[I-1].topP[k];
     glTexCoord2f(r1/lim,j/(float) L.segs);
     glVertex3f(Vpos.x,Vpos.y,Vpos.z);
    }
   glEnd();
  }
 }
}

// priprava 2D billboard textur ********************************************
void PlantTree::RenderSprite(int S)
{
 glPushMatrix();

 // vymazanie zasobnikov
 glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

 // DOROBIT: celu tuto funkciu je potrebne upravit tak, aby sa 2D textura
 // maximalne podobala 3D stromu, iba tak dosiahneme zelany efekt,
 // momentalne su prechody medzi 3D a 2D dost viditelne co narusa
 // realny vnem
 glTranslatef(0.0f,-5.0f,-8.8f);
 glRotatef(-10.0f,1.0f,0.0f,0.0f);
 glRotatef((float) -(S*(360/SPRITES_NUM)),0.0f,1.0f,0.0f);

 // vykreslenie 3D stromu
 RenderTree();

 // kopia obrazovky do objektu textury
 glGenTextures(1,&sprite_texture[S]);
 glBindTexture(GL_TEXTURE_2D,sprite_texture[S]);

 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);

 glCopyTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,104,0,512,512,0);

 glPopMatrix();
}

 // vykreslenie stromu *****************************************************
void PlantTree::Draw(float PX,float PY,float PZ)
{
 Vpos.x=PX; Vpos.y=PY; Vpos.z=PZ;
 dist=Vpos.Vlength();
 // ak je pozorovatel blizsie ako je hranicna vzdialenost, vykresli sa 3D strom
 if (dist<THRESHOLD1)
  // vykreslenie 3D stromu
  RenderTree();
 else if (dist<THRESHOLD3) { // inak sa vykresli billboard
  glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
  glEnable(GL_BLEND);

  // ziskanie smeroveho vektora poloha pozorovatela -> 0 (poloha stromu)
  Vdir=-(Vpos-tVec3(0.0f,0.0f,0.0f));
  // INFO: poloha pozorovatela
  //fprintf(stderr,"%f %f %f\n",Vdir.x,Vdir.y,Vdir.z);
  Vdir.y=0.0f; // prvok Y nie je dolezity
  Vdir.Vnormalize();
  // uhol medzi smerovym vektorom a osou Z
  r=(Vdir.dotP(tVec3(0.0f,0.0f,1.0f)));
  // INFO: uhol
  //fprintf(stderr,"%f\n",r);
  // ziskanie uhlu v stupnoch
  if (Vdir.x>0.0f) Vrot.y=RAD2DEG(acosf(r));
  else Vrot.y=360.0f-RAD2DEG(acosf(r));
  // INFO: uhol v stupnoch
  //fprintf(stderr,"%f\n",Vrot.y);
  // korekcia posledneho uhla
  if (Vrot.y+(float) (180/SPRITES_NUM)>=360.0f) Vrot.y-=360.0f;
  // vyber konkretnej textury
  j=0;
  for (i=0;i<SPRITES_NUM;i++)
   if (Vrot.y+(float) (180/SPRITES_NUM)>(float) (360/SPRITES_NUM*i) &&
       Vrot.y+(float) (180/SPRITES_NUM)<=(float) (360/SPRITES_NUM*(i+1)))
    { glBindTexture(GL_TEXTURE_2D,sprite_texture[i]); j=i; }

  // DOROBIT: v idealnom pripade by sa mali bykreslit 2-3 billboardy,
  // ktore by mali spravnu priesvitnost a spolu by vytvarali najkorektnejsi
  // vzhlad stromu, ten billboard, ktory je viditelny "priamo" by mal byt najviac
  // viditelny, ostatne by ho mali v urcenom pomere (priesvitnosti) doplnat
  // zamedzili by sme tak skokovym zmenam zobrazenia stromu
  /*glPushMatrix();
  if ((j+1)<SPRITES_NUM) glBindTexture(GL_TEXTURE_2D,sprite_texture[j+1]);
  else glBindTexture(GL_TEXTURE_2D,sprite_texture[0]);
  // natocenie billboardu
  glRotatef(Vrot.y+(float) (360/SPRITES_NUM),0.0f,1.0f,0.0f);
  // farba billboardu - nastavenie priesvitnosti
  if (dist>THRESHOLD2) glColor4f(1.0f,1.0f,1.0f,(1.0f-(dist-THRESHOLD2)/(THRESHOLD3-THRESHOLD2))/3.0f);
  else glColor4f(1.0f,1.0f,1.0f,0.33f);
  // vykreslenie billboardu
  glBegin(GL_QUADS);
   glTexCoord2f(0.0f,0.0f); glVertex3f(-5.0f, 0.0f,0.0f);
   glTexCoord2f(1.0f,0.0f); glVertex3f( 5.0f, 0.0f,0.0f);
   glTexCoord2f(1.0f,1.0f); glVertex3f( 5.0f,10.0f,0.0f);
   glTexCoord2f(0.0f,1.0f); glVertex3f(-5.0f,10.0f,0.0f);
  glEnd();
  glPopMatrix();
  glPushMatrix();
  if ((j-1)>=0) glBindTexture(GL_TEXTURE_2D,sprite_texture[j-1]);
  else glBindTexture(GL_TEXTURE_2D,sprite_texture[7]);
  // natocenie billboardu
  glRotatef(Vrot.y-(float) (360/SPRITES_NUM),0.0f,1.0f,0.0f);
  // farba billboardu - nastavenie priesvitnosti
  if (dist>THRESHOLD2) glColor4f(1.0f,1.0f,1.0f,(1.0f-(dist-THRESHOLD2)/(THRESHOLD3-THRESHOLD2))/3.0f);
  else glColor4f(1.0f,1.0f,1.0f,0.33f);
  // vykreslenie billboardu
  glBegin(GL_QUADS);
   glTexCoord2f(0.0f,0.0f); glVertex3f(-5.0f, 0.0f,0.0f);
   glTexCoord2f(1.0f,0.0f); glVertex3f( 5.0f, 0.0f,0.0f);
   glTexCoord2f(1.0f,1.0f); glVertex3f( 5.0f,10.0f,0.0f);
   glTexCoord2f(0.0f,1.0f); glVertex3f(-5.0f,10.0f,0.0f);
  glEnd();
  glPopMatrix();*/
  glPushMatrix();
  // natocenie billboardu
  glRotatef(Vrot.y,0.0f,1.0f,0.0f);
  // farba billboardu - nastavenie priesvitnosti
  if (dist>THRESHOLD2) glColor4f(1.0f,1.0f,1.0f,(1.0f-(dist-THRESHOLD2)/(THRESHOLD3-THRESHOLD2))/1.0f);
  else glColor4f(1.0f,1.0f,1.0f,1.0f);
  // vykreslenie billboardu
  // DOROBIT: velkost stvoruholnika je dana "pevne", tento pristup nie je najvhodnejsi,
  // bolo by spravnejsie urcit velkost stvoruholnika na zaklade patricnych vztahov
  glBegin(GL_QUADS);
   glTexCoord2f(0.0f,0.0f); glVertex3f(-5.6f,-0.1f,0.0f);
   glTexCoord2f(1.0f,0.0f); glVertex3f( 5.6f,-0.1f,0.0f);
   glTexCoord2f(1.0f,1.0f); glVertex3f( 5.6f,10.9f,0.0f);
   glTexCoord2f(0.0f,1.0f); glVertex3f(-5.6f,10.9f,0.0f);
  glEnd();
  glPopMatrix();

  // standardna farba - nepriesvitna
  glColor4f(1.0f,1.0f,1.0f,1.0f);
  glDisable(GL_BLEND);
 }
}
