#ifndef _MPLANTTREE_H_
#define _MPLANTTREE_H_

// systemove kniznice
#include <vector>

// matematika
#include "m3dmath.h"

// pocet billboard textur
#define SPRITES_NUM 8

// hranicna vzdialenost pre vykreslenie 3D stromu
#define THRESHOLD1 40.0f
// hranicna vzdialenost pre vykreslenie 2D billboardu
#define THRESHOLD2 60.0f
// hranicna vzdialenost pre vykreslenie stromu
#define THRESHOLD3 80.0f

using namespace std;

// kostra stromu
struct skeleton
{
 // zaciatok a koniec ciary
 tVec3 s,e;
 // hlbka rekurzie
 int d;
 // dlzka ciary
 float l;
 // indikator ci ma dana cast kostry stromu priameho "rodica"
 bool is_parent;
 // pocet segmentov ciary
 int segs;
 // prirastky zmien poloh zaciatkov a koncov jednotlivych segmentov
 // je mozne vytvorit max. 3 segmenty!!!
 tVec3 sgs[3];
 // maximalne 8 vrcholov, z ktorych sa vytvori horny koniec
 // danej casti modelu stromu
 tVec3 topP[8];
};

// konar stromu
struct branch
{
 // konstruktor
 branch() { left=middle=right=NULL; }
 // destruktor
 ~branch() { }

 // startovacia poloha konara, dlzka, uhol natocenia okolo osi Z
 float start,length,angle;
 // lavy, stredny a pravy "potomok" konara
 branch *left,*middle,*right;
 // matica obsahujuca polohu a natocenie konara
 tMatrix4 m;
};

// list stromu
struct leaf
{
 tVec3 pos,dir; // poloha, natocenie
 tVec3 bend;    // ohnutie listu
};

// trieda premennych a funkcii pre vytvorenie 3D stromu
class PlantTree {
public:
 // konstruktor
 PlantTree();
 // destructor
 ~PlantTree();

 // vytvorenie stromu
 int Generate(int MAXDEPTH,
              float LENGTH,float LENGTHMOD,
              float ANGLE,float ANGLEMOD,
              float BRANCHWIDTH,float LEAFSIZE,
              GLuint BRANCHTEX,GLuint LEAFTEX);
 // vykreslenie kostry stromu
 void DrawSkeleton(void);
 // vykreslenie stromu
 void Draw(float PX,float PY,float PZ);
private:
 // pomocne premenne
 int i,j,k,l,mem;
 float r,lim,inc,r1,inc1;
 // pomocne vektory
 tVec3 Vpos,Vrot,Vdir,Vdirnorm,Vup,Vright,Vstart,Vend,Vbend;
 tVec3 PA,PB,PC,PD;
 float Vsize;
 // pomocna matica
 float mat[16];
 // matica 4x4
 tMatrix4 mv_matrix;
 // pomocne vektory
 tVec3 VA,VB;
 // identifikatory textur pre konare (aj kmen), listy a billboard stromu
 GLuint branch_texture,leaf_texture,sprite_texture[SPRITES_NUM];
 // hlbka rekurzie
 int Tmaxdepth;
 // dlzka casti kostry stromu, koeficient kratenia dlzky
 float Tlength,Tlengthmod;
 // uhol natocenia casti kostry stromu, koeficient zmeny natocenia
 float Tangle,Tanglemod;
 // hrubka konara, velkost listu
 float Tbranchwidth,Tleafsize;
 // pole ciar - casti kostry stromu a pole listov stromu
 vector<skeleton> lines;
 vector<leaf> leaves;
 // kmen stromu - 1. cast kostry stromu
 branch *stem;
 // ciara patriaca kostre stromu
 skeleton Sline,L;
 // listy stromu
 leaf lf1,lf2;
 // velkost spodneho a vrchneho konca konara
 float botrad,toprad;
 // indikacia existencie "rodica" casti kostry stromu
 bool parent;
 // vzdialenost od stromu
 float dist;

 // vytvorenie jednotlivych casti kostry stromu
 // REKURZIVNA FUNKCIA!!!
 int TreeRecurse(branch *B,tMatrix4 M,
                 int DEPTH,
                 float LENGTH,float ANGLE);
 // uvolnenie vyhradeneho pamatoveho miesta
 // REKURZIVNA FUNKCIA!!!
 int RecurseDelete(branch *B,int DEPTH);
 // vykreslenie 3D stromu
 void RenderTree();
 // vykreslenie konara
 void RenderBranch(unsigned int I,float BOTRAD,float TOPRAD,float SEGS);
 // priprava 2D billboard textur
 void RenderSprite(int S);
};

#endif
